from cmath import sqrt
from random import randint


# --------------------- #

'''Criar vetor'''
# nulo
# unitario
# aleatorio
# oposto

'''Imprimir vetor'''

'''Igualdade de vetores'''

'''Calculos com vetores'''
# subtração
# soma
# produto escalar
# multiplicação por escalar
# divisao por escalar
# modulo

# --------------------- #


class Vetor:

    # metodos

    # o unico parametro é a dimensão, R², R³ ou maior

    # se não passar nenhum parametro, retorna vetor de R²
    def criar_vetor_nulo(self, dimensao=2):
            vector = [0] * dimensao
            return vector

    def criar_vetor_unitario(self, dimensao=2):
            vector = [1] * dimensao
            return vector

    # se não passar parametro retorna vetor do R² preenchido de forma aleatoria, entre 0 a 9
    def criar_vetor_aleatorio(self, dimensao=2):
            vector = []
            for i in range(dimensao):
                vector.append(randint(0, 9))
            return vector

    # para o vetor oposto tem que passar como parametro o vetor que quer o oposto dele
    def criar_vetor_oposto(self, vector):
        vetor_oposto = []
        for i in range(len(vector)):
            vetor_oposto.append(-vector[i])
        return vetor_oposto

    # imprime os vetores como a gente escreve no caderno
    # aceita a passagem de vários vetores de vetores em casa posição, se booleano igual a True, ou de varios vetores se for False
    def imprimir_vetor(self, plural=False, *args):
        if (plural):
            for linha_de_linha in range(len(args)):
                for linha in range(len(args[linha_de_linha])):
                    print('(', end='')
                    for coluna in range(len(args[0][linha])):
                        if (coluna == (len(args[0][linha]) - 1)):
                            print(str(args[0][linha][coluna]) + ')', end='\n')
                        else:
                            print(str(args[0][linha][coluna]) + ',', end=' ')
        else:
            for linha in range(len(args)):
                print('(', end='')
                for coluna in range(len(args[linha])):
                    if (coluna == (len(args[linha]) - 1)):
                        print(str(args[linha][coluna]) + ')', end='\n')
                    else:
                        print(str(args[linha][coluna]) + ',', end=' ')

    # aceita a passagem de mais de um vetor porém só compara dois vetores
    def igualdade_de_vetores(self, *args):
        tamanho = 0
        if (len(args[0]) == len(args[1])):
            for item in range(len(args[0])):
                if args[0][item] == args[1][item]:
                    tamanho += 1
                else:
                    break
            if(tamanho == len(args[1])):
                print("Vetores iguais", end='\n')
            else:
                print("Os vetores não são iguais", end='\n')
        else:
            print("Vetores de dimensões diferentes, então não são iguais", end='\n')

    # subtração de vetores do mesmo tamanho, não aceita vetor de vetores
    def subtracao_de_vetor(self, *args):
        for i in range(len(args) - 1):
            if (len(args[i]) != len(args[i + 1])):
                print("Não subtrai vetores de tamanhos diferentes", end='\n')
                return 0

        vetor_subtracao = []
        elemento = 0
        for linha in range(len(args)):
            for coluna in range(len(args[linha])):
                if ((coluna and elemento) == 0):
                    elemento = args[coluna][linha]
                else:
                    elemento -= args[coluna][linha]
            vetor_subtracao.append(elemento)
        return vetor_subtracao

    # soma de vetores do mesmo tamanho, não aceita vetor de vetores
    def soma_de_vetor(self, *args):
        for i in range(len(args) - 1):
            if (len(args[i]) != len(args[i + 1])):
                print("Não soma vetores de tamanhos diferentes", end='\n')
                return 0

        vetor_soma = []
        elemento = 0
        for linha in range(len(args)):
            for coluna in range(len(args[linha])):
                if ((coluna and elemento) == 0):
                    elemento = args[coluna][linha]
                else:
                    elemento += args[coluna][linha]
            vetor_soma.append(elemento)
        return vetor_soma

    # multiplica cada posição dos vetores e soma com a próxima, x*x + y*y + z*z...
    def produto_escalar_vetor(sef, *args):
        for i in range(len(args) - 1):
            if (len(args[i]) != len(args[i + 1])):
                print("Não calcula o produto escalar de vetores de tamanhos diferentes", end='\n')
                return 0

        produto_escalar = 0
        multiplicacao = 0
        for items in range(len(args[0])):
            for elementos in range(len(args)):
                if (elementos == 0):
                    multiplicacao = args[elementos][items]
                else:
                    multiplicacao *= args[elementos][items]
            produto_escalar += multiplicacao
        return produto_escalar

    # ao inves de retornar, altera os valores dos vetores passados por parâmetro
    def multiplicacao_por_escalar_vetor(self, *args, escalar):
        for items in range(len(args)):
            for elementos in range(len(args[items])):
                args[items][elementos] *= escalar

    # ao inves de retornar, altera os valores dos vetores passados por parâmetro
    def divisao_por_escalar_vetor(self, *args, escalar):
        for items in range(len(args)):
            for elementos in range(len(args[items])):
                args[items][elementos] /= escalar

    # retorna uma lista com o módulo de cada vetor que foi passado como parametro nos seus respectivos indices
    def modulo_vetor(self, *args):
        modulo = []
        quadrado = 0
        for items in range(len(args)):
            for elementos in range(len(args[items])):
                if (elementos == 0):
                    quadrado = args[items][elementos] ** 2
                else:
                    quadrado += args[items][elementos] ** 2
            modulo.append(sqrt(quadrado))
        return modulo

    '''def angulo_entre_vetor(self, vetor1, vetor2, print=False, returner=True):
        if (len(vetor1) == len(vetor2)):
            cima = Vetor.produto_escalar_vetor(vetor1, vetor2)
            baixo = Vetor.modulo_vetor(vetor1) * Vetor.modulo_vetor(vetor2)
            resultado = cima / baixo
            if (print):
                print(f'{math.degrees(math.acos(resultado))} graus')
            if (returner):
                return math.degrees(math.acos(resultado))

        else:
            print("Não há como calcular vetores de dimensões diferentes", end='\n')'''


# class Matriz:


# -----TESTES----- #

ve = Vetor()
vetor1 = ve.criar_vetor_unitario(4)
ve.imprimir_vetor(vetor1)
vetor2 = ve.criar_vetor_oposto(vetor1)
ve.imprimir_vetor(vetor2)



from gettext import NullTranslations
import math
from operator import le
from random import randint
from math import sqrt

# Operações com Matrizes #

'''Criar Matriz'''
# nula
# unitaria
# random
# identidade

'''Imprimir'''
# imprime

'''Igualdade'''
# igualdade

'''Calculos com Matrizes'''
# soma
# subtração
# produto escalar
# multiplicação por escalar
# potencia
# traco
# determinante
# transposta
# simetrica


# ----------------------- #

def cria_matriz_aleatoria(linha, coluna):
    matriz = []
    for x in range(linha):
        partes = []
        for y in range(coluna):
            partes.append(randint(0, 9))
        matriz.append(partes)

    return matriz


def cria_matriz_nula(linha, coluna):
    matriz = []
    for x in range(linha):
        partes = []
        for y in range(coluna):
            partes.append(0)
        matriz.append(partes)

    return matriz


def cria_matriz_unitaria(linha, coluna):
    matriz = []
    for x in range(linha):
        partes = []
        for y in range(coluna):
            partes.append(1)
        matriz.append(partes)

    return matriz


def cria_matriz_identidade(tamanho):
    matriz = []
    for x in range(tamanho):
        partes = []
        for y in range(tamanho):
            if x == y:
                partes.append(1)
            else:
                partes.append(0)
        matriz.append(partes)

    return matriz


def imprime_matriz(vetor):
    for x in range(len(vetor)):
        print('|', end='')
        for y in range(len(vetor[0])):
            print(vetor[x][y], end=" ")
        print('|', end='')
        print()
    print()


def igualdade_matriz(matriz1, matriz2):
    tamanho = 0
    items = len(matriz1) * len(matriz1[0])
    if ((len(matriz1) == len(matriz2)) and (len(matriz1[0]) == len(matriz2[0]))):
        for l in range(len(matriz1)):
            for c in range(len(matriz1[0])):
                if (matriz1[l][c] == matriz2[l][c]):
                    tamanho += 1
                else:
                    break

        if(tamanho == items):
            print("Matrizes iguais", end='\n')
        else:
            print("As matrizes não são iguais", end='\n')
    else:
        print("Matrizes de dimensões diferentes", end='\n')


def soma_matriz(a, b):
    if (len(a) == len(b) and len(a[0]) == len(b[0])):
        tamanho_linha = len(a)
        tamanho_coluna = len(a[0])
        matriz = cria_matriz_nula(tamanho_linha, tamanho_coluna)
        for x in range(tamanho_linha):
            for y in range(tamanho_coluna):
                matriz[x][y] = a[x][y] + b[x][y]
        return matriz

    else:
        print("Não há como calcular matrizes de dimensões diferentes", end='\n')


def subtracao_matriz(a, b):
    if (len(a) == len(b) and len(a[0]) == len(b[0])):
        tamanho_linha = len(a)
        tamanho_coluna = len(a[0])
        matriz = cria_matriz_nula(tamanho_linha, tamanho_coluna)
        for x in range(tamanho_linha):
            for y in range(tamanho_coluna):
                matriz[x][y] = a[x][y] - b[x][y]
        return matriz

    else:
        print("Não há como calcular matrizes de dimensões diferentes", end='\n')


def multiplicacao_matriz_por_escalar(matriz, escalar):
    tamanho_linha = len(matriz)
    tamanho_coluna = len(matriz[0])
    for x in range(tamanho_linha):
        for y in range(tamanho_coluna):
            matriz[x][y] = matriz[x][y] * escalar
    return matriz


def produto_escalar_matriz(matriz1, matriz2):
    tamanho_coluna = len(matriz1[0])
    tamanho_linha = len(matriz2)
    if (tamanho_coluna == tamanho_linha):
        matriz3 = cria_matriz_nula(len(matriz1), len(matriz2[0]))
        for li in range(len(matriz1)):
            for c in range(len(matriz2[0])):
                for seg in range(tamanho_coluna):
                    matriz3[li][c] += matriz1[li][seg] * matriz2[seg][c]
        return matriz3

    else:
        print("Não é possível", end='\n')


def potencia_matriz(matriz, expoente):

    tamanho_linha = len(matriz)
    tamanho_coluna = len(matriz[0])
    
    resultado = matriz
    
    if (tamanho_linha != tamanho_coluna):
        print("Só há como calular matriz quadrada", end='\n')
    
    elif expoente == 0:
        return cria_matriz_identidade(tamanho_linha)

    elif (expoente == 1):
        return matriz

    elif (expoente >= 2):
        for i in range(1, expoente):
            resultado = produto_escalar_matriz(resultado, matriz)
        return resultado

    else:
        print("Não há como calcular matriz de expoente negativo")


def traco_matriz(matriz):
    traco = 0
    tamanho_linha = len(matriz)
    tamanho_coluna = len(matriz[0])
    for l in range(tamanho_linha):
        for c in range(tamanho_coluna):
            if l == c:
                traco += matriz[l][c]
    return traco


'''def determinante(matriz):
    final = len(matriz[0]) - 1
    diagonal_principal = 1
    diagonal_secundaria = 1
    for l in range(len(matriz)):
        for c in range(len(matriz[0])):
            if l == c:
                diagonal_principal *= matriz[l][c]
                diagonal_secundaria *= matriz[l][final]
                final -= 1
    return diagonal_principal - diagonal_secundaria'''


def transposta(matriz):
    linha = len(matriz[0])
    coluna = len(matriz)
    matriz_transposta = cria_matriz_nula(linha, coluna)
    for lin in range(linha):
        for col in range(coluna):
            matriz_transposta[lin][col] = matriz[col][lin]
    return matriz_transposta


def matriz_simetrica(matriz1, matriz2):
    saida = False
    for linha in range(len(matriz1)):
        for coluna in range(len(matriz1[0])):
            if(matriz1[linha][coluna] == matriz2[coluna][linha]):
                continue
            else:
                saida = True
                print("Não são simétricas", end="\n")
                break

        if (saida):
            break
    if(not saida):
        print("São simétricas", end="\n")



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    page_description = f.read()

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

setup(
    name="Algebra_Linear",
    version="0.0.1",
    author="Dirceu Silvestre",
    author_email="dirceu_silvestre@hotmail.com",
    description="Biblioteca com funções que geram e calculam vetores e matrizes",
    long_description="",
    long_description_content_type="",
    url="https://github.com/DirceuSilvestre/Biblioteca-Algebra-Linear",
    packages=find_packages(),
    install_requires=requirements,
    python_requires='>=3.8',
)